@@ -12,7 +12,6 @@
 
 import json
 import random
-import string
 import subprocess
 
 from oslotest import base
@@ -47,6 +46,25 @@ class BaseTestCase(base.BaseTestCase):
         else:
             return result
 
+    def rand_name(self, name='', prefix=None):
+        """Generate a random name that includes a random number
+
+        :param str name: The name that you want to include
+        :param str prefix: The prefix that you want to include
+        :return: a random name. The format is
+                 '<prefix>-<name>-<random number>'.
+                 (e.g. 'prefixfoo-namebar-154876201')
+        :rtype: string
+        """
+        # NOTE(lajos katona): This method originally is in tempest-lib.
+        randbits = str(random.randint(1, 0x7fffffff))
+        rand_name = randbits
+        if name:
+            rand_name = name + '-' + rand_name
+        if prefix:
+            rand_name = prefix + '-' + rand_name
+        return rand_name
+
     def assertCommandFailed(self, message, func, *args, **kwargs):
         signature = [func]
         signature.extend(args)
@@ -59,14 +77,16 @@ class BaseTestCase(base.BaseTestCase):
                 message, e.output,
                 'Command "%s" fails with different message' % e.cmd)
 
-    def resource_provider_create(self, name=''):
+    def resource_provider_create(self,
+                                 name='',
+                                 parent_provider_uuid=None):
         if not name:
-            random_part = ''.join(random.choice(string.ascii_letters)
-                                  for i in range(10))
-            name = RP_PREFIX + random_part
+            name = self.rand_name(name='', prefix=RP_PREFIX)
 
-        res = self.openstack('resource provider create ' + name,
-                             use_json=True)
+        to_exec = 'resource provider create ' + name
+        if parent_provider_uuid is not None:
+            to_exec += ' --parent-provider ' + parent_provider_uuid
+        res = self.openstack(to_exec, use_json=True)
 
         def cleanup():
             try:
@@ -80,8 +100,10 @@ class BaseTestCase(base.BaseTestCase):
 
         return res
 
-    def resource_provider_set(self, uuid, name):
+    def resource_provider_set(self, uuid, name, parent_provider_uuid=None):
         to_exec = 'resource provider set ' + uuid + ' --name ' + name
+        if parent_provider_uuid is not None:
+            to_exec += ' --parent-provider ' + parent_provider_uuid
         return self.openstack(to_exec, use_json=True)
 
     def resource_provider_show(self, uuid, allocations=False):
@@ -92,7 +114,8 @@ class BaseTestCase(base.BaseTestCase):
         return self.openstack(cmd, use_json=True)
 
     def resource_provider_list(self, uuid=None, name=None,
-                               aggregate_uuids=None, resources=None):
+                               aggregate_uuids=None, resources=None,
+                               in_tree=None):
         to_exec = 'resource provider list'
         if uuid:
             to_exec += ' --uuid ' + uuid
@@ -103,6 +126,8 @@ class BaseTestCase(base.BaseTestCase):
                 '--aggregate-uuid %s' % a for a in aggregate_uuids)
         if resources:
             to_exec += ' ' + ' '.join('--resource %s' % r for r in resources)
+        if in_tree:
+            to_exec += ' --in-tree ' + in_tree
 
         return self.openstack(to_exec, use_json=True)
 
@@ -115,12 +140,18 @@ class BaseTestCase(base.BaseTestCase):
             use_json=True
         )
 
-    def resource_allocation_set(self, consumer_uuid, allocations):
+    def resource_allocation_set(self, consumer_uuid, allocations,
+                                project_id=None, user_id=None,
+                                use_json=True):
         cmd = 'resource provider allocation set {allocs} {uuid}'.format(
             uuid=consumer_uuid,
             allocs=' '.join('--allocation {}'.format(a) for a in allocations)
         )
-        result = self.openstack(cmd, use_json=True)
+        if project_id:
+            cmd += ' --project-id %s' % project_id
+        if user_id:
+            cmd += ' --user-id %s' % user_id
+        result = self.openstack(cmd, use_json=use_json)
 
         def cleanup(uuid):
             try:
@@ -169,6 +200,12 @@ class BaseTestCase(base.BaseTestCase):
         return self.openstack('resource provider usage show ' + uuid,
                               use_json=True)
 
+    def resource_show_usage(self, project_id, user_id=None):
+        cmd = 'resource usage show %s' % project_id
+        if user_id:
+            cmd += ' --user-id %s' % user_id
+        return self.openstack(cmd, use_json=True)
+
     def resource_provider_aggregate_list(self, uuid):
         return self.openstack('resource provider aggregate list ' + uuid,
                               use_json=True)
@@ -236,3 +273,12 @@ class BaseTestCase(base.BaseTestCase):
     def resource_provider_trait_delete(self, uuid):
         cmd = 'resource provider trait delete %s ' % uuid
         self.openstack(cmd)
+
+    def allocation_candidate_list(self, resources, required=None, limit=None):
+        cmd = 'allocation candidate list ' + ' '.join(
+            '--resource %s' % resource for resource in resources)
+        if required is not None:
+            cmd += ''.join([' --required %s' % t for t in required])
+        if limit is not None:
+            cmd += ' --limit %d' % limit
+        return self.openstack(cmd, use_json=True)
